import sys
import unittest
import launchpad
from settings import *
from mock import *
import launchpadTrigger


class LaunchpadTriggerInstantiation(unittest.TestCase):
    def test_instatiation(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            l = launchpad.LaunchpadTrigger()
            launchpadAgent.login.assert_called_once_with()
            l._launchpad.people.assert_called_once_with(LAUNCHPAD_LOGIN)


class LaunchpadTriggerTestCase(unittest.TestCase):

    launchpadTrigger = None
    jenkinsUserName = 'jenkins-user'

    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        user = MagicMock()
        user.name = self.jenkinsUserName
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()
            self.launchpadTrigger._launchpad_user = user

    def test_new_mp_candidate_revision_in_queue(self):
            #60 is the revision which was validated as last
            self.launchpadTrigger.get_latest_review = MagicMock(return_value=60)
            mp = MagicMock()
            #this is the current branch revision
            mp.source_branch.revision_count = 61
            #latest_candidate_validated should return False as there is a new
            #revision in the queue (61)
            self.assertFalse(self.launchpadTrigger.latest_candidate_validated(mp))

    def test_mp_already_validated(self):
            #60 is the revision which was validated as last
            self.launchpadTrigger.get_latest_review = MagicMock(return_value=60)
            mp = MagicMock()
            #this is the current branch revision
            mp.source_branch.revision_count = 60
            #latest_candidate_validated should return True. This rev was
            #already validated
            self.assertTrue(self.launchpadTrigger.latest_candidate_validated(mp))


    def test_get_latest_review_no_comments(self):
        mp = MagicMock()
        mp.all_comments = []
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 0)


    def test_get_latest_review_unrelated_comment(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = 'unrelated comment'
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 0)


    def test_get_latest_review_which_passed(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.author.name = self.jenkinsUserName
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = "PASSED: Continuous integration, rev:47\nURL:..."
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 47)

    def test_get_latest_review_which_failed(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.author.name = self.jenkinsUserName
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = "FAILED: Continuous integration, rev:47\nURL:..."
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 47)


    def test_testing_not_in_progress_approved(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        review = launchpad.MergeProposalReview(mp)
        review.unlock()
        self.assertFalse(self.launchpadTrigger.testing_in_progress(mp))


    def test_testing_not_in_progress_abstain(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        review = launchpad.MergeProposalReview(mp)
        review.lock()
        self.assertTrue(self.launchpadTrigger.testing_in_progress(mp))

    def test_user_not_allowed_to_trigger_job(self):
        mp = MagicMock()
        mp.registrant.name = 'random-user'
        membership = MagicMock()
        membership.team.name = 'random-team'
        mp.registrant.memberships_details = [membership]
        self.assertFalse(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))


    def test_user_member_of_allowed_team(self):
        mp = MagicMock()
        mp.registrant.name = 'random-user'
        membership = MagicMock()
        membership.team.name = 'mrazik'
        mp.registrant.memberships_details = [membership]
        self.assertTrue(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))

    def test_user_allowed_to_trigger(self):
        mp = MagicMock()
        mp.registrant.name = 'mrazik'
        membership = MagicMock()
        membership.team.name = ''
        mp.registrant.memberships_details = [membership]
        self.assertTrue(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))

    def test_start_jenkins_job(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        j = MagicMock()
        j.get_job_info = MagicMock(return_value = {'buildable' : True})
        jenkins = MagicMock()
        jenkins.Jenkins = MagicMock(return_value = j)
        with patch('launchpad.jenkins',new=jenkins):
            self.launchpadTrigger.start_jenkins_job('url','jenkins_job',mp)
            self.assertEqual(j.build_job.call_count,1)
            self.assertEqual(mp.nominateReviewer.call_count,1)

    def test_start_nonbuildable_jenkins_job(self):
        mp = MagicMock()
        j = MagicMock()
        j.get_job_info = MagicMock(return_value = {'buildable' : False})
        jenkins = MagicMock()
        jenkins.Jenkins = MagicMock(return_value = j)
        with patch('launchpad.jenkins',new=jenkins):
            self.assertEqual(self.launchpadTrigger.start_jenkins_job('url','jenkins_job',mp), False)
            self.assertEqual(j.build_job.call_count,0)
            self.assertEqual(mp.nominateReviewer.call_count,0)
            self.assertEqual(mp.createComment.call_count,0)




class LaunchpadTriggerCIWithWrongUser(unittest.TestCase):
    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()

        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=False)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_with_wrong_user(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)

    def test_trigger_ci_with_no_mps(self):
         self.launchpadTrigger.trigger_ci_building([],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIWhileTestingInProgress(unittest.TestCase):
    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()

        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=True)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_testing_in_progress(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIWhileRevisionValidated(unittest.TestCase):
    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()

        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.latest_candidate_validated = MagicMock(return_value=True)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_Revision_validated(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIW(unittest.TestCase):
    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()

        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.latest_candidate_validated = MagicMock(return_value=False)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_Revision_validated(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,1)



